(* ::Package:: *)

(* ::Input:: *)
(*Clear["Global`*"]*)


(* ::Title:: *)
(*MD Benchmark Code*)


(* ::Text:: *)
(*The program is divided in four chapters:*)
(**)
(*1. REBO Potentials.*)
(*In this chapter the form of the REBO potential to be tested is assigned. In the section "2nd generation Brenner potential", the default setting for this potential is implemented, according to [2]; in particular, in the subsection "Potential components" the components introduced in (17) are specified.  In the section "Analytical discrete model'' the definition of the nanostresses (19) is implemented; this definition is independent of the REBO potential one choses.*)
(**)
(*2. Armchair CNTs*)
(*In this chapter the equilibrium problem for armchair CNTs is solved. In the section "Generalities'' the geometric conditions on the order parameters are established and the nanostresses are computed. In the section "Solution of the equilibrium equations'' the solution of the system 3_1 and  4_1 is determined as a function of the applied force F and the chiral number n. In the section "Radius'' the natural radius is computed as a function of F and n and then determined for F=0, namely in the natural configuration. In the section "Energy" the natural energy is computed as a function of F and n and then determined for F=0, namely the cohesive energy. In the section "Young's modulus'' the current and the referential lengths of a CNT are determined, and the strain measure is defined, as a function of F and n; on introducing the nominal thickness, the Young's modulus is defined as a function of F and n, and then computed for a tiny value of F, up to convergence. In section "Poisson coefficient'', the named material parameter is defined as a function of F and n, and then computed for a tiny value of F, up to convergence.*)
(**)
(*3. Zigzag CNTs.*)
(*This chapter has the same section as the previous one, but implemented for the zigzag case; the different geometric constraints are properly included.*)
(**)
(*4. Summary of Results.*)
(*In this chapter the benchmark solutions are collected for the visualization in the CDF file Benchmark_solutions.cdf.*)
(**)
(*Equation numbers and bibliographic reference numbers are those of the article associated to this code.*)
(**)


(* ::Chapter:: *)
(*1. REBO Potentials*)


(* ::Section:: *)
(*2nd generation Brenner potential*)


(* ::Subsection:: *)
(*G Function*)


(* ::Text:: *)
(*According to the definition in reference [2]*)


d1={Subscript[d, 0]-> 0.375449999999999,Subscript[d, 1]->1.406776475152104,
Subscript[d, 2]->2.254377494462425,Subscript[d, 3]->2.031282890266629,
Subscript[d, 4]->1.429711740681566,Subscript[d, 5]->0.502401399437276};
G1[\[Theta]_]=\!\(
\*SubsuperscriptBox[\(\[Sum]\), \(i = 0\), \(5\)]\(
\*SubscriptBox[\(d\), \(i\)]*
\*SuperscriptBox[\(Cos[\[Theta]]\), \(i\)]\)\)/.d1;

d2={Subscript[d, 0]->0.707277245734129,Subscript[d, 1]->5.677435848489921,
Subscript[d, 2]->24.09701877750397,Subscript[d, 3]->57.59183195999613,
Subscript[d, 4]->71.88287000288395,Subscript[d, 5]->36.27886067346856};
G2[\[Theta]_]=\!\(
\*SubsuperscriptBox[\(\[Sum]\), \(i = 0\), \(5\)]\(
\*SubscriptBox[\(d\), \(i\)]*
\*SuperscriptBox[\(Cos[\[Theta]]\), \(i\)]\)\)/.d2;

d3={Subscript[d, 0]->-0.644399999999948,Subscript[d, 1]->-6.207999999999618,
Subscript[d, 2]->-20.05899999999891,Subscript[d, 3]->-30.22799999999852,
Subscript[d, 4]->-21.72399999999905,Subscript[d, 5]->-5.990399999999763};
G3[\[Theta]_]=\!\(
\*SubsuperscriptBox[\(\[Sum]\), \(i = 0\), \(5\)]\(
\*SubscriptBox[\(d\), \(i\)]*
\*SuperscriptBox[\(Cos[\[Theta]]\), \(i\)]\)\)/.d3;
G[\[Theta]_]=Piecewise[{{G1[\[Theta]],0<=\[Theta]<0.6082*\[Pi]},{G2[\[Theta]],0.6082*\[Pi]<=\[Theta]<2/3*\[Pi]},{G3[\[Theta]],2/3*\[Pi]<=\[Theta]<=\[Pi]}}];


(* ::Input:: *)
(**)


(* ::Subsection:: *)
(*Potential parameters*)


ev=0.1602176487;(*unit conversion constant*)


potparameters={Subscript[B, 1]-> 12388.79197798,Subscript[B, 2]->17.56740646509,
Subscript[B, 3]->30.71493208065, Subscript[\[Beta], 1]->4.7204523127,Subscript[\[Beta], 2]->1.4332132499,
Subscript[\[Beta], 3]->1.3826912506,Q->0.3134602960833,A->10953.544162170,\[Alpha]0->4.7465390606595};


T=-0.004048375;


(* ::Subsection:: *)
(*Potential components*)


(* ::Text:: *)
(*Equation (17)*)
(*(Reference  [2])*)


VA[r_]=-\!\(
\*SubsuperscriptBox[\(\[Sum]\), \(n = 1\), \(3\)]\(
\*SubscriptBox[\(B\), \(n\)]*
\*SuperscriptBox[\(E\), \(\(-
\*SubscriptBox[\(\[Beta]\), \(n\)]\)*r\)]\)\)/.potparameters;
VR[r_]=(1+Q/r)*A*E^(-\[Alpha]0*r)/.potparameters;
DVA[r_]=D[VA[r],r];
DVR[r_]=D[VR[r],r];
ba=(1+2*G[\[Beta]])^(-(1/2))+2*T*(1-Cos[\[CapitalTheta]1]^2);
bb=(1+G[\[Alpha]]+G[\[Beta]])^(-(1/2))+T*(2*(1-Cos[\[CapitalTheta]2]^2)+(1-Cos[\[CapitalTheta]3]^2)+(1-Cos[\[CapitalTheta]4]^2));


(* ::Input:: *)
(**)


(* ::Section:: *)
(*Analytical discrete model*)


(* ::Subsection:: *)
(*Geometry  *)


n1[n_]:=2*n;
n2[n_]:=n;


(* ::Subsection:: *)
(*Nanostresses*)


(* ::Text:: *)
(*Equation (19)*)


\[Sigma]a=DVR[a]+ba*DVA[a];
\[Sigma]b=DVR[b]+bb*DVA[b];
\[Tau]\[Alpha]=D[bb,\[Alpha]]*VA[b];
\[Tau]\[Beta]=1/4*(D[ba,\[Beta]]*VA[a]+2*D[bb,\[Beta]]*VA[b]);
T1=1/2*D[ba,\[CapitalTheta]1]*VA[a];
T2=1/2*D[bb,\[CapitalTheta]2]*VA[b];
T3=D[bb,\[CapitalTheta]3]*VA[b];
T4=D[bb,\[CapitalTheta]4]*VA[b];


(* ::Chapter:: *)
(*2. Armchair CNTs*)


(* ::Section:: *)
(*Generalities*)


(* ::Subsection:: *)
(*Geometric constraints*)


(* ::Text:: *)
(*Reference [1]*)


\[Phi][n_]:=\[Pi]/n1[n];


\[Beta][n_]:=ArcCos[-Cos[\[Alpha]/2]*Cos[\[Phi][n]]];
\[CapitalTheta]1[n_]:=2*ArcSin[(Cos[\[Alpha]/2]*Sin[\[Phi][n]])/Sin[\[Beta][n]]];
\[CapitalTheta]2[n_]:=ArcSin[Sin[\[Phi][n]]/Sin[\[Beta][n]]];
\[CapitalTheta]3[n_]:=2*\[CapitalTheta]2[n];
\[CapitalTheta]4[n_]:=0;


\[Sigma]an[n_]:=\[Sigma]a/.{\[Beta]->\[Beta][n],\[CapitalTheta]1->\[CapitalTheta]1[n],\[CapitalTheta]2->\[CapitalTheta]2[n],\[CapitalTheta]3->\[CapitalTheta]3[n],\[CapitalTheta]4->\[CapitalTheta]4[n]}
\[Sigma]bn[n_]:=\[Sigma]b/.{\[Beta]->\[Beta][n],\[CapitalTheta]1->\[CapitalTheta]1[n],\[CapitalTheta]2->\[CapitalTheta]2[n],\[CapitalTheta]3->\[CapitalTheta]3[n],\[CapitalTheta]4->\[CapitalTheta]4[n]}
\[Tau]\[Alpha]n[n_]:=\[Tau]\[Alpha]/.{\[Beta]->\[Beta][n],\[CapitalTheta]1->\[CapitalTheta]1[n],\[CapitalTheta]2->\[CapitalTheta]2[n],\[CapitalTheta]3->\[CapitalTheta]3[n],\[CapitalTheta]4->\[CapitalTheta]4[n]}
\[Tau]\[Beta]n[n_]:=\[Tau]\[Beta]/.{\[Beta]->\[Beta][n],\[CapitalTheta]1->\[CapitalTheta]1[n],\[CapitalTheta]2->\[CapitalTheta]2[n],\[CapitalTheta]3->\[CapitalTheta]3[n],\[CapitalTheta]4->\[CapitalTheta]4[n]}
T1n[n_]:=T1/.{\[Beta]->\[Beta][n],\[CapitalTheta]1->\[CapitalTheta]1[n],\[CapitalTheta]2->\[CapitalTheta]2[n],\[CapitalTheta]3->\[CapitalTheta]3[n],\[CapitalTheta]4->\[CapitalTheta]4[n]}
T2n[n_]:=T2/.{\[Beta]->\[Beta][n],\[CapitalTheta]1->\[CapitalTheta]1[n],\[CapitalTheta]2->\[CapitalTheta]2[n],\[CapitalTheta]3->\[CapitalTheta]3[n],\[CapitalTheta]4->\[CapitalTheta]4[n]}
T3n[n_]:=T3/.{\[Beta]->\[Beta][n],\[CapitalTheta]1->\[CapitalTheta]1[n],\[CapitalTheta]2->\[CapitalTheta]2[n],\[CapitalTheta]3->\[CapitalTheta]3[n],\[CapitalTheta]4->\[CapitalTheta]4[n]}
T4n[n_]:=T4/.{\[Beta]->\[Beta][n],\[CapitalTheta]1->\[CapitalTheta]1[n],\[CapitalTheta]2->\[CapitalTheta]2[n],\[CapitalTheta]3->\[CapitalTheta]3[n],\[CapitalTheta]4->\[CapitalTheta]4[n]}


(* ::Section:: *)
(*Solution of the equilibrium equations*)


solA[F_,n_]:=FindRoot[{\[Sigma]an[n]==0,\[Sigma]bn[n]-F/n1[n]*Sin[\[Alpha]/2]==0,\[Tau]\[Alpha]n[n]+
2*D[\[Beta][n],\[Alpha]]*\[Tau]\[Beta]n[n]+D[\[CapitalTheta]1[n],\[Alpha]]*T1n[n]+2*D[\[CapitalTheta]2[n],\[Alpha]]*T2n[n]+D[\[CapitalTheta]3[n],\[Alpha]]*T3n[n]
-1/2*F/n1[n]*b*Cos[\[Alpha]/2]==0},{{a,1.4},{b,1.4},{\[Alpha],2}}]


(* ::Section:: *)
(*Radius*)


(* ::Text:: *)
(*Reference [1]*)


\[Rho]a[n_]:=(b/2*Cos[\[Alpha]/2]+a/2*Cos[\[Phi][n]])/Sin[\[Phi][n]];
\[Rho][n_]:=Sqrt[\[Rho]a[n]^2+a^2/4];
rA[F_,n_]:=\[Rho][n]/10/.solA[F,n];(*radius in nm*)


(* ::Subsection:: *)
(*Natural radius (output in Table 3)*)


\[Rho]A[n_]:=rA[0,n]


(* ::Section:: *)
(*Energy *)


Va=VR[a]+ba*VA[a];
Vb=VR[b]+bb*VA[b];


V[F_,n_]:=(Va+2*Vb)/2/.{\[Beta]->\[Beta][n],\[CapitalTheta]1->\[CapitalTheta]1[n],\[CapitalTheta]2->\[CapitalTheta]2[n],\[CapitalTheta]3->\[CapitalTheta]3[n],\[CapitalTheta]4->\[CapitalTheta]4[n]}/.solA[F,n]


(* ::Input:: *)
(**)


(* ::Subsection:: *)
(*Cohesive energy (output of Table 3)*)


(* ::Text:: *)
(*Reference [1]*)


EnergyA[n_]:=V[0,n](*ev/atom*)


(* ::Section:: *)
(*Young Modulus*)


(* ::Text:: *)
(*Equations (13)-(14)*)


\[Lambda]A[n_]:=2*Sin[\[Alpha]/2]*n2[n]*b;(*current length*)
\[Lambda]0A[n_]:=\[Lambda]A[n]/.solA[0,n];(*referential length*)
\[Epsilon]A[n_]:=(\[Lambda]A[n]-\[Lambda]0A[n])/\[Lambda]0A[n];(*strain measure*)
eA[F_,n_]:=\[Epsilon]A[n]/.solA[F,n];


(* ::Input:: *)
(**)


t=0.34(*nm*);(*nominal thickness*)


YA[F_,n_]:=(F/eA[F,n])*ev*(1/(2*\[Pi]*rA[F,n]*t))*10(*Young modulus, GPa*)


(* ::Subsection:: *)
(*Young modulus in the origin (output in Table 4)*)


YoungA[n_]:=If[n<20,YA[10^-10,n],YA[10^-9,n]](*GPa*)


(* ::Section:: *)
(*Poisson coefficient*)


(* ::Text:: *)
(*Equation (15)*)


\[Nu]A[F_,n_]:=-((rA[F,n]-rA[0,n])/rA[0,n])*1/eA[F,n]


(* ::Subsection:: *)
(*Poisson coefficient in the origin (output in Table 4)*)


PoissonA[n_]:=If[n<20,\[Nu]A[10^-10,n],\[Nu]A[10^-9,n]]


(* ::Chapter:: *)
(*3. Zigzag CNTs*)


(* ::Section:: *)
(*Generalities*)


(* ::Subsection:: *)
(*Geometric constraints*)


(* ::Text:: *)
(*Reference [1]*)


\[Phi]z[n_]:=\[Pi]/n;


\[Beta]z[n_]:=\[Pi]-ArcSin[Sin[\[Alpha]/2]/Cos[\[Phi]z[n]/2]];
\[CapitalTheta]1z[n_]:=\[Phi]z[n];
\[CapitalTheta]2z[n_]:=ArcSin[(Sin[\[Beta]z[n]]*Sin[\[Phi]z[n]])/Sin[\[Alpha]]];
\[CapitalTheta]3z[n_]:=0;
\[CapitalTheta]4z[n_]:=2*\[CapitalTheta]2z[n];


\[Sigma]az[n_]:=\[Sigma]a/.{\[Beta]->\[Beta]z[n],\[CapitalTheta]1->\[CapitalTheta]1z[n],\[CapitalTheta]2->\[CapitalTheta]2z[n],\[CapitalTheta]3->\[CapitalTheta]3z[n], \[CapitalTheta]4->\[CapitalTheta]4z[n]}
\[Sigma]bz[n_]:=\[Sigma]b/.{\[Beta]->\[Beta]z[n],\[CapitalTheta]1->\[CapitalTheta]1z[n],\[CapitalTheta]2->\[CapitalTheta]2z[n],\[CapitalTheta]3->\[CapitalTheta]3z[n], \[CapitalTheta]4->\[CapitalTheta]4z[n]}
\[Tau]\[Alpha]z[n_]:=\[Tau]\[Alpha]/.{\[Beta]->\[Beta]z[n],\[CapitalTheta]1->\[CapitalTheta]1z[n],\[CapitalTheta]2->\[CapitalTheta]2z[n],\[CapitalTheta]3->\[CapitalTheta]3z[n], \[CapitalTheta]4->\[CapitalTheta]4z[n]}
\[Tau]\[Beta]z[n_]:=\[Tau]\[Beta]/.{\[Beta]->\[Beta]z[n],\[CapitalTheta]1->\[CapitalTheta]1z[n],\[CapitalTheta]2->\[CapitalTheta]2z[n],\[CapitalTheta]3->\[CapitalTheta]3z[n], \[CapitalTheta]4->\[CapitalTheta]4z[n]}
T1z[n_]:=T1/.{\[Beta]->\[Beta]z[n],\[CapitalTheta]1->\[CapitalTheta]1z[n],\[CapitalTheta]2->\[CapitalTheta]2z[n],\[CapitalTheta]3->\[CapitalTheta]3z[n], \[CapitalTheta]4->\[CapitalTheta]4z[n]}
T2z[n_]:=T2/.{\[Beta]->\[Beta]z[n],\[CapitalTheta]1->\[CapitalTheta]1z[n],\[CapitalTheta]2->\[CapitalTheta]2z[n],\[CapitalTheta]3->\[CapitalTheta]3z[n], \[CapitalTheta]4->\[CapitalTheta]4z[n]}
T3z[n_]:=T3/.{\[Beta]->\[Beta]z[n],\[CapitalTheta]1->\[CapitalTheta]1z[n],\[CapitalTheta]2->\[CapitalTheta]2z[n],\[CapitalTheta]3->\[CapitalTheta]3z[n], \[CapitalTheta]4->\[CapitalTheta]4z[n]}
T4z[n_]:=T4/.{\[Beta]->\[Beta]z[n],\[CapitalTheta]1->\[CapitalTheta]1z[n],\[CapitalTheta]2->\[CapitalTheta]2z[n],\[CapitalTheta]3->\[CapitalTheta]3z[n], \[CapitalTheta]4->\[CapitalTheta]4z[n]}


(* ::Section:: *)
(*Solution of the equilibrium equations*)


solZ[F_,n_]:=FindRoot[{\[Sigma]az[n]-F/n2[n]==0,\[Sigma]bz[n]+F/(2*n2[n])*Cos[\[Beta]z[n]]==0,
\[Tau]\[Alpha]z[n]+2*D[\[Beta]z[n],\[Alpha]]*\[Tau]\[Beta]z[n]+2*D[\[CapitalTheta]2z[n],\[Alpha]]*T2z[n]+D[\[CapitalTheta]4z[n],\[Alpha]]*T4z[n]
-1/2*F/n2[n]*b*D[\[Beta]z[n],\[Alpha]]*Sin[\[Beta]z[n]]==0},{{a,1.4},{b,1.4},{\[Alpha],2}}]


(* ::Section:: *)
(*Radius*)


(* ::Text:: *)
(*Reference [1]*)


rz[n_]:=Sin[\[Beta]z[n]]/(2*Sin[\[Phi]z[n]/2])*b;
r[F_,n_]:=rz[n]/10/.solZ[F,n];(*radius in nm*)


(* ::Subsection:: *)
(*Natural radius (output in Table 3)*)


\[Rho]Z[n_]:=r[0,n](*nm*)


(* ::Section:: *)
(*Energy*)


VZ[F_,n_]:=(Va+2*Vb)/2/.{\[Beta]->\[Beta]z[n],\[CapitalTheta]1->\[CapitalTheta]1z[n],\[CapitalTheta]2->\[CapitalTheta]2z[n],\[CapitalTheta]3->\[CapitalTheta]3z[n], \[CapitalTheta]4->\[CapitalTheta]4z[n]}/.solZ[F,n]


(* ::Subsection:: *)
(*Cohesive energy (output in Table 3)*)


EnergyZ[n_]:=VZ[0,n](*ev/atom*)


(* ::Section:: *)
(*Young Modulus*)


(* ::Text:: *)
(*Equations (13)-(14)*)


\[Lambda]Z[n_]:=(1-b/a*Cos[\[Beta]z[n]])*n1[n]*a;(*current length*)
\[Lambda]0Z[n_]:=\[Lambda]Z[n]/.solZ[0,n];(*referential length*)
\[Epsilon]Z[n_]:=(\[Lambda]Z[n]-\[Lambda]0Z[n])/\[Lambda]0Z[n];(*strain measure*)
eZ[F_,n_]:=\[Epsilon]Z[n]/.solZ[F,n];


YZ[F_,n_]:=(F/eZ[F,n])*ev*(1/(2*\[Pi]*r[F,n]*t))*10(*Young modulus,GPa*)


(* ::Subsection:: *)
(*Young modulus in the origin (output in Table 4)*)


YoungZ[n_]:=YZ[10^-8,n](*GPa*)


(* ::Section:: *)
(*Poisson ratio*)


(* ::Text:: *)
(*Equation (15)*)


\[Nu][F_,n_]:=-((r[F,n]-r[0,n])/r[0,n])*1/eZ[F,n]


(* ::Subsection:: *)
(*Poisson coefficient in the origin (output in Table 4)*)


PoissonZ[n_]:=\[Nu][10^-8,n]


(* ::Section:: *)
(*4. Summary of Results*)


Radius[0,n_]:=\[Rho]A[n]
Radius[1,n_]:=\[Rho]Z[n]
Energy[0,n_]:=EnergyA[n]
Energy[1,n_]:=EnergyZ[n]
Young[0,n_]:=YoungA[n]
Young[1,n_]:=YoungZ[n]
Poisson[0,n_]:=PoissonA[n]
Poisson[1,n_]:=PoissonZ[n]


(* ::Subsection:: *)
(*Creation of the shell for cdf file*)


Manipulate[
Column[{ NumberForm[Radius[q,n],4]  "radius [nm]:",NumberForm[Energy[q,n],5] "cohesive energy [ev/atom]:",
 NumberForm[Young[q,n],7] "Young modulus [GPa]:", NumberForm[Poisson[q,n],4] "Poisson coefficient:"},
Left,", "],
{{q,0,"show"},{0->"armchair",1->"zigzag"},Setter},
{n,3,30},
AutorunSequencing->{1,2},
TrackedSymbols:>{q,n},
ControlPlacement->Top,
SaveDefinitions->True,
ControlType->InputField]
